package co.uk.fues.freudbot;

import co.uk.fues.freudbot.NLP.NLP;
import co.uk.fues.freudbot.NLP.Phenomise;
import co.uk.fues.freudbot.NLP.TemporalDepParser;
import co.uk.fues.freudbot.cluster.KMeans;
import co.uk.fues.freudbot.data.TweetCleaner;
import com.google.common.util.concurrent.AtomicDouble;
import org.apache.commons.io.FileUtils;
import org.apache.commons.math3.stat.inference.TTest;
import org.deeplearning4j.models.paragraphvectors.ParagraphVectors;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.text.DecimalFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import static co.uk.fues.freudbot.NLP.TemporalDepParser.joinedResultsConvoFileName;
import static co.uk.fues.freudbot.NLP.TemporalDepParser.joinedResultsFileName;

public class Experiments {

    public static void main(String [] a) throws InterruptedException, IOException, URISyntaxException {

        //        TwitterAPI twitterAPI = new TwitterAPI();
        //        twitterAPI.runDeclarationCollection(dataDirectory);

        cluster("", "/joined.txt", "/tweetresultsplainclean2.txt", "/tweetresultsphemnome.txt", "/tweetBatchResults/res/");
        clusterConvo();
        reflexivePronounAudit();
        musicality();
        vocab();
        temporalPronounAudit();
        modals();
        aggression();
        //cluster("/data", "/joined.txt", "/tweetresultsplainclean2.txt", "/tweetresultsphemnome.txt", "/tweetBatchResults/res/");
        //removeIf("/tweetresultsplainclean2.txt");
    }


    // Cluster the declarations
    public static void cluster(String dataDirectory, String joinedResultsFileName, String cleanResultsFileName, String phenomeFileName, String nlpBatchDir) throws IOException, URISyntaxException, InterruptedException {


        String resultsFileName = "/tweetresultsplain2.txt";
        TweetCleaner tweetCleaner = new TweetCleaner();
        tweetCleaner.makeLines(dataDirectory, resultsFileName);
        tweetCleaner.cleanFile(resultsFileName, cleanResultsFileName);

        LinkedList<String> text =  NLP.getData(cleanResultsFileName);
        NLP.batchRun(text, nlpBatchDir);
        NLP.stitch(nlpBatchDir, joinedResultsFileName);

        Phenomise.postProcessPhenomise(cleanResultsFileName, phenomeFileName);

        String resultsDir = "/results/";

        KMeans kMeans = new KMeans();
        ParagraphVectors vectors = KMeans.getParagraphVectors(phenomeFileName);
        kMeans.method(vectors, cleanResultsFileName, resultsDir, 5);
        kMeans.method(vectors, cleanResultsFileName, resultsDir, 10);
        kMeans.method(vectors, cleanResultsFileName, resultsDir, 20);
        kMeans.method(vectors, cleanResultsFileName, resultsDir, 30);
        kMeans.showClusters(resultsDir, 5);
        kMeans.showClusters(resultsDir, 10);
        kMeans.showClusters(resultsDir, 20);
        kMeans.showClusters(resultsDir, 30);

    }

    public static void clusterConvo() throws IOException, URISyntaxException, InterruptedException {


        String resultsFileName = "/tweetconvoresultsplain2.txt";
        String cleanResultsFileName = "/tweetconvoresultsplainclean2.txt";
        String dataDirName = "/conversations";
        TweetCleaner tweetCleaner = new TweetCleaner();
        tweetCleaner.makeLines(dataDirName, resultsFileName);
        tweetCleaner.cleanFile(resultsFileName, cleanResultsFileName);

        String joinedResultsFileName = "/joinedconvo.txt";
        LinkedList<String> text = NLP.getData(cleanResultsFileName);
        String nlpBatchDir = "/tweetconvoBatchResults/res/";
        NLP.batchRun(text, nlpBatchDir);
        NLP.stitch(nlpBatchDir, joinedResultsFileName);

        String phenomeFileName = "/tweetconvoresultsphemnome.txt";
        Phenomise.postProcessPhenomise(cleanResultsFileName, phenomeFileName);

        String resultsDir = "/resultsconvo/";

        KMeans kMeans = new KMeans();
        ParagraphVectors vectors = KMeans.getParagraphVectors(phenomeFileName);
        kMeans.method(vectors, cleanResultsFileName, resultsDir, 5);
        kMeans.method(vectors, cleanResultsFileName, resultsDir, 10);
        kMeans.method(vectors, cleanResultsFileName, resultsDir, 20);
        kMeans.method(vectors, cleanResultsFileName, resultsDir, 30);
        kMeans.showClusters(resultsDir, 5);
        kMeans.showClusters(resultsDir, 10);
        kMeans.showClusters(resultsDir, 20);
        kMeans.showClusters(resultsDir, 30);

    }


    // Reflexive pronoun use
    // does it differs in the thread?

    public static void vocab() throws IOException {
        String cleanResultsFileName = "/tweetresultsplainclean2.txt";
        LinkedList<String> botTweetText = NLP.getData(cleanResultsFileName);
        Set<String> words = new HashSet<>();
        AtomicInteger total = new AtomicInteger();
        botTweetText.forEach(line-> {
                List<String> w = Arrays.asList(line.replaceAll(",", "").toLowerCase().split(" "));
                words.addAll(w);
                total.set(total.get() + w.size() - 3);
            }
        );

        String cleanConvoResultsFileName = "/tweetconvoresultsplainclean2.txt";
        List<String> convotext = NLP.getData(cleanConvoResultsFileName);
        convotext = convotext.subList(0, botTweetText.size());
        Set<String> wordsconvo = new HashSet<>();
        AtomicInteger totalw = new AtomicInteger();
        convotext.forEach(line->
                {
                    List<String> w = Arrays.asList(line.replaceAll(",", "").toLowerCase().split(" "));
                    wordsconvo.addAll(w);
                    totalw.set(totalw.get() + w.size() - 3);
                }
        );
        System.out.println(words.size() + " " + wordsconvo.size());
    }

    public static void reflexivePronounAudit() throws IOException {


        String cleanResultsFileName = "/tweetresultsplainclean2.txt";

        LinkedList<String> botTweetText = NLP.getData(cleanResultsFileName);
        String rpBatchDir = "/tweetBatchResults/rp/";
        //ReflexivePronoun.run(botTweetText, rpBatchDir);

        String cleanConvoResultsFileName = "/tweetconvoresultsplainclean2.txt";

        LinkedList<String> convotext = NLP.getData(cleanConvoResultsFileName);
        String convorpBatchDir = "/tweetconvoBatchResults/rp/";
        //ReflexivePronoun.run(convotext, convorpBatchDir);

        LinkedList<String> botPronounOnlyText = NLP.getData(rpBatchDir + "rp.txt");
        LinkedList<String> rpConvotext = NLP.getData(convorpBatchDir + "rp.txt");

        List<String> rplist = Arrays.asList(new String[]{"i", "you", "we", "us", "me", "i'll", "you'll", "we'll", "i'd", "you'd", "we'd", "i'm", "you're", "we're", "ourselves", "yourself", "myself", "they", "they'll", "they'd", "they're", "themselves"});
        List<String> youlist = Arrays.asList(new String[]{"you", "you'll", "you'd", "you're", "yourself"});
        List<String> ilist = Arrays.asList(new String[]{"i", "i'll", "i'd", "i'm", "myself"});
        List<String> welist = Arrays.asList(new String[]{"we", "we'll", "we'd", "we're", "ourselves"});
        List<String> theylist = Arrays.asList(new String[]{"they", "they'll", "they'd", "they're", "themselves"});

        runAudit(botTweetText, convotext, botPronounOnlyText, rpConvotext, rplist);
        runAudit(botTweetText, convotext, botPronounOnlyText, rpConvotext, youlist);
        runAudit(botTweetText, convotext, botPronounOnlyText, rpConvotext, ilist);
        runAudit(botTweetText, convotext, botPronounOnlyText, rpConvotext, welist);
        runAudit(botTweetText, convotext, botPronounOnlyText, rpConvotext, theylist);


    }


    private static void runAudit(LinkedList<String> botTweetText, LinkedList<String> convotext, LinkedList<String> botPronounOnlyText, LinkedList<String> rpConvotext, List<String> rplist) {
        AtomicDouble rpWC = new AtomicDouble();
        AtomicInteger tweetWC = new AtomicInteger();
        List<Double> avgs1 = new ArrayList<>();

        for(int i = 0; i< botTweetText.size(); i++) {
            String tweetText = botTweetText.get(i).toLowerCase();
            String rpText = botPronounOnlyText.get(i);
            String [] splits = tweetText.split(" ");
            String [] rpSplits = rpText.split(" ");
            int altCount = 0;
            for(String w:splits) {
                if(rplist.contains(w)) {
                    altCount++;
                }
            }


            double density = (altCount) / (((double) splits.length));
            if(density==1 || density<0) {
                density = 0;
            }
            rpWC.getAndAdd(density);
            tweetWC.getAndIncrement();
            avgs1.add(density);
        }


        double avg = rpWC.get() / tweetWC.get();

        rpWC = new AtomicDouble();
        tweetWC = new AtomicInteger();
        List<Double> avgs2 = new ArrayList<>();

        for(int i = 0; i< convotext.size(); i++) {
            String tweetText = convotext.get(i).toLowerCase();
            String rpText = rpConvotext.get(i);
            String [] splits = tweetText.split(" ");
            String [] rpSplits = rpText.split(" ");
            int altCount = 0;
            for(String w:splits) {
                if(rplist.contains(w)) {
                    altCount++;
                }
            }
            //double density = Math.abs(((double) rpSplits.length) / ((double) splits.length));
            double density = altCount / ((double) splits.length);
            if(density==1) {
                density = 0;
            }
            rpWC.getAndAdd(density);
            tweetWC.getAndIncrement();
            avgs2.add(density);
        }
        ;

        double cavg = rpWC.get() / tweetWC.get();
        TTest tTest = new TTest();

        boolean avg1Bigger = avgs1.size()>avgs2.size();
        List<Double> avgs1trim = avg1Bigger? avgs1.subList(0, avgs2.size()):avgs1;
        List<Double> avgs2trim = avg1Bigger? avgs2:avgs2.subList(0, avgs1.size());

        double [] a = avgs1trim.stream().mapToDouble(i->i).toArray();
        double [] b = avgs2trim.stream().mapToDouble(i->i).toArray();

        double t = tTest.pairedT(a, b);
        avg = avg * 100;
        cavg = cavg * 100;
        DecimalFormat df=new DecimalFormat("#.##");
        System.out.println("declaration: " + df.format(avg) + " conversation: " +  df.format(cavg) + " diff: " + " t " + t);
    }

    public static void temporalPronounAudit() throws IOException {
        TemporalDepParser.main(null);
        LinkedList<String> declarationText = NLP.getData(joinedResultsFileName);
        LinkedList<String> convoText = NLP.getData(joinedResultsConvoFileName);

        Map<String, Integer> counts = new HashMap<>();
        for(String line:declarationText) {
            String [] tenses = line.split(" ");
            for (String tense:tenses) {
                counts.putIfAbsent(tense, 0);
                counts.put(tense, counts.get(tense)+1);
            }
        }
        System.out.println("declaration");
        counts.entrySet().forEach(stringIntegerEntry -> {
            System.out.println(stringIntegerEntry.getKey() + " : " + stringIntegerEntry.getValue());
        });
        counts = new HashMap<>();
        for(String line:convoText) {
            String [] tenses = line.split(" ");
            for (String tense:tenses) {
                counts.putIfAbsent(tense, 0);
                counts.put(tense, counts.get(tense)+1);
            }
        }
        System.out.println("convo");
        double adjust = (double)declarationText.size()/(double)convoText.size();
        counts.entrySet().forEach(stringIntegerEntry -> {
            System.out.println(stringIntegerEntry.getKey() + " : " + (int)((double)stringIntegerEntry.getValue()*adjust));
        });

    }

    public static void modals() throws IOException {
        String cleanResultsFileName = "/tweetresultsplainclean2.txt";
        LinkedList<String> declarationText = NLP.getData(cleanResultsFileName);
        String cleanConvoResultsFileName = "/tweetconvoresultsplainclean2.txt";
        List<String> convoText = NLP.getData(cleanConvoResultsFileName);

        List<String> pastModal = Arrays.asList(new String[]{"would have", "could have", "may have", "might have", "should have", "must have", "would not have", "could not have", "may not have", "might not have", "should not have", "must not have"});
        List<String> futureAnterior = Arrays.asList(new String[]{"ll have", "ll not have"});

        int decPastModal = 0;
        int decFutureAnterior = 0;
        for (String text:declarationText) {
            for (String past:pastModal) {
                if(text.contains(past)) {
                    decPastModal++;
                }
            }
            for (String fut:futureAnterior) {
                if(text.contains(fut)) {
                    decFutureAnterior++;
                }
            }

        }

        int conPastModal = 0;
        int conFutureAnterior = 0;
        for (String text:convoText) {
            for (String past:pastModal) {
                if(text.contains(past)) {
                    conPastModal++;
                }
            }
            for (String fut:futureAnterior) {
                if(text.contains(fut)) {
                    conFutureAnterior++;
                }
            }

        }

        double adjust = (double)declarationText.size()/(double)convoText.size();

        System.out.println("dec past " + decPastModal + " dec " + decFutureAnterior);
        System.out.println("conv past " + ((double)conPastModal) * adjust + " fut " + ((double)conFutureAnterior)*adjust);
    }

    // Study the rupture, the defiance of meaning, grammar, the failure score
    // score the tweet in convo, is there a change?

    // Find teh change patterns in speech upon the declaration
    // before and after, any similarities

    // musicality
    public static void musicality() throws IOException {

        String cleanResultsFileName = "/tweetresultsphemnome.txt";

        LinkedList<String> botTweetText = NLP.getData(cleanResultsFileName);

        String cleanConvoResultsFileName = "/tweetconvoresultsphemnome.txt";

        LinkedList<String> convotext = NLP.getData(cleanConvoResultsFileName);

        double botscore = 0.0;
        List<Double> botscores = new ArrayList<>();

        Map<String, Integer> sylls = new HashMap<>();
        for (String text : botTweetText) {
            botscore = botscore + syllableScore(text, sylls);
            botscores.add(syllableScore(text, null));
        }
        botscore = botscore/botTweetText.size();

        double convscore = 0.0;
       List<Double> convscores = new ArrayList<>();

        Map<String, Integer> syllsc = new HashMap<>();
        for (String text : convotext) {
            convscore = convscore + syllableScore(text, syllsc);
            convscores.add(syllableScore(text, null));
        }
        convscore = convscore/convotext.size();

        TTest tTest = new TTest();

        convscores = convscores.subList(0, botscores.size());
        double [] a = botscores.stream().mapToDouble(i->i).toArray();
        double [] b = convscores.stream().mapToDouble(i->i).toArray();
        double t = tTest.pairedT(a, b);

        System.out.println(botscore + " " + convscore + " t " + t);
    }

    public static void aggression() throws IOException {

            String cleanResultsFileName = "/tweetresultsplainclean2.txt";
            LinkedList<String> declarationText = NLP.getData(cleanResultsFileName);
            String cleanConvoResultsFileName = "/tweetconvoresultsplainclean2.txt";
            List<String> convoText = NLP.getData(cleanConvoResultsFileName);
            convoText = convoText.subList(0, declarationText.size());

            List<String> aggressionWords = FileUtils.readLines(new File(System.getProperty("user.dir") + "/swear_words.txt"));

            int decPastModal = 0;
            int wc = 0;
            for (String text:declarationText) {
                for (String past:aggressionWords) {
                    if(text.toLowerCase().contains(past)) {
                        decPastModal++;
                    }
                }
                wc = text.split(" ").length - 4;

            }

            int conPastModal = 0;
            int cwc = 0;
            for (String text:convoText) {
                for (String past:aggressionWords) {
                    if(text.toLowerCase().contains(past)) {
                        conPastModal++;
                    }
                }
                cwc = text.split(" ").length;
            }

            double adjust = (double)wc/(double)cwc;

            System.out.println("dec aggression " + decPastModal );
            System.out.println("conv aggression " + ((double)conPastModal) * adjust );
        }

    public static double syllableScore(String text, Map<String, Integer> sylls) {
        String [] syllables = text.split(" ");
        List<String> listw = Arrays.stream(syllables).filter(w -> w.length()>0).map(String::toLowerCase).collect(Collectors.toList());
        List<String> list = Arrays.stream(syllables).filter(w -> w.length()>1).map(syl -> {
            String ns = syl.toLowerCase();
            if(sylls!=null) {
                sylls.putIfAbsent(ns, 1);
                sylls.put(ns, sylls.get(ns) + 1);
            }
            return ns;
        }).collect(Collectors.toList());
        double score = ((double) listw.size())/new HashSet<String>(list).size();

        return score;
    }
}
