package co.uk.fues.freudbot.NLP;

import com.johnsnowlabs.nlp.DocumentAssembler;
import com.johnsnowlabs.nlp.RecursivePipeline;
import com.johnsnowlabs.nlp.annotators.Tokenizer;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.spark.ml.PipelineModel;
import org.apache.spark.ml.PipelineStage;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Encoders;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;
import scala.collection.mutable.WrappedArray;

import java.io.File;
import java.io.IOException;
import java.util.*;

public class Phenomise {

    public static void main(String [] args) throws IOException {
        String cleanResultsFileName = "/joined.txt";
        String phenomeFileName = "/tweetresultsphemnome.txt";
        postProcessPhenomise(cleanResultsFileName, phenomeFileName);
    }
    public static void postProcessPhenomise(String cleanResultsFileName, String phenomeFileName) throws IOException {
        SparkSession spark = com.johnsnowlabs.nlp.SparkNLP.start(false, false, false, "12G");
        LinkedList<String> text = new LinkedList<String>();
        text.addAll(FileUtils.readLines(new File(System.getProperty("user.dir") + cleanResultsFileName)));

        Dataset<Row> data = spark.createDataset(text, Encoders.STRING()).toDF("text");

        DocumentAssembler document = new DocumentAssembler();
        document.setInputCol("text");
        document.setOutputCol("document");
        document.setCleanupMode("disabled");

        Tokenizer tokenizer = new Tokenizer();
        tokenizer.setInputCols(new String[] {"document"});
        tokenizer.setOutputCol("token");

        RecursivePipeline pipeline = new RecursivePipeline();
        pipeline.setStages(new PipelineStage[] {document, tokenizer});
        PipelineModel pipelineModel = pipeline.fit(data);

        Dataset<Row> transformed = pipelineModel.transform(data);

        text.clear();
        Object res = transformed.collect();
        Row[] rows = (Row[]) res;
        Map<String, String> wordToPhoneme = getPhenomeMap();

        List<String> phenomised = new ArrayList<>();
        for(Row row:rows) {
            String tweetText = (String) row.get(0);
            WrappedArray tokenResult = (WrappedArray) row.get(2);
            Row[] tokensRow = (Row[]) tokenResult.array();
            //String depres = "";
            for (Row tokenVal:tokensRow) {
                String token = (String) tokenVal.get(3);
                tweetText = phenomiseWord(tweetText, token, wordToPhoneme);
            }
            phenomised.add(tweetText);
        }
        FileUtils.writeLines(new File(System.getProperty("user.dir") + phenomeFileName), phenomised);
    }

    private static String phenomiseWord(String tweetText, String token, Map<String, String> wordToPhoneme) {
        if(wordToPhoneme.containsKey(token.toUpperCase().replaceAll("’", "'"))) {
            String replace = wordToPhoneme.get(token.toUpperCase());
            if(StringUtils.isAllLowerCase(token)) {
                replace = replace.toLowerCase();
            } else if(StringUtils.isAllUpperCase(String.valueOf(token.charAt(0)))) {
                replace = StringUtils.capitalize(replace);
            }
            try {

                if(token.equals(".") || token.equals("?")  || token.equals("...")  || token.equals("-")) {
                    tweetText = tweetText.replaceAll("//"+token, " " + replace.replaceAll(" ", ""));
                } else if(token.equals(",") || token.equals("!")) {
                    tweetText = tweetText.replaceAll(token, " " + replace.replaceAll(" ", ""));
                } else {
                    tweetText = tweetText.replaceAll(token, replace);//.replaceAll(" ", ""));
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            if(!token.equalsIgnoreCase("@")) {
                System.out.println(token);
            }
        }
        return tweetText;
    }

    private static Map<String, String> getPhenomeMap() throws IOException {
        List<String> lines = FileUtils.readLines(new File(System.getProperty("user.dir") + "/phonetics.txt"));
        Map<String, String> wordToPhoneme = new HashMap<>();
        for (String line:lines) {
            String [] splits = line.split(" ");
            String w = splits[0];
            String [] ph = Arrays.copyOfRange(splits, 1, splits.length);
            String concat = "";
            for (int i=0;i<ph.length;i++) {
                concat = concat + ph[i];
                if(i< ph.length-1) {
                    concat = concat+" ";
                }
            }
            wordToPhoneme.put(w, concat);
            wordToPhoneme.put(",", "COMMA");
            wordToPhoneme.put("?", "QUESTION");
            wordToPhoneme.put("??", "QUESTION");
            wordToPhoneme.put("???", "QUESTION");
            wordToPhoneme.put("!", "EXCLAMATION");
            wordToPhoneme.put("!!", "EXCLAMATION");
            wordToPhoneme.put("!!!", "EXCLAMATION");
            wordToPhoneme.put(".", "DOT");
            wordToPhoneme.put("..", "ELLIPSES");
            wordToPhoneme.put("...", "ELLIPSES");
            wordToPhoneme.put("....", "ELLIPSES");
            wordToPhoneme.put(".....", "ELLIPSES");
            wordToPhoneme.put("......", "ELLIPSES");
            wordToPhoneme.put(".......", "ELLIPSES");
            wordToPhoneme.put("-", "DASH");
            wordToPhoneme.put("BOT", "B AH0 T");

        }
        return wordToPhoneme;
    }

}
