package co.uk.fues.freudbot.NLP;

import com.johnsnowlabs.nlp.DocumentAssembler;
import com.johnsnowlabs.nlp.RecursivePipeline;
import com.johnsnowlabs.nlp.annotators.Tokenizer;
import com.johnsnowlabs.nlp.annotators.ner.dl.NerDLModel;
import com.johnsnowlabs.nlp.annotators.parser.dep.DependencyParserModel;
import com.johnsnowlabs.nlp.annotators.pos.perceptron.PerceptronModel;
import com.johnsnowlabs.nlp.annotators.sbd.pragmatic.SentenceDetector;
import com.johnsnowlabs.nlp.embeddings.WordEmbeddingsModel;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.spark.ml.PipelineModel;
import org.apache.spark.ml.PipelineStage;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Encoders;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;
import scala.collection.mutable.WrappedArray;

import java.io.File;
import java.io.IOException;
import java.util.*;

public class ReflexivePronoun {

    public static void main(String [] args) throws IOException {
        String cleanResultsFileName = "/joined.txt";
        String phenomeFileName = "/tweetresultsphemnome.txt";
    }
    public static List<String> run(LinkedList<String> text, String baseResultsFileName) throws IOException {
        SparkSession spark = com.johnsnowlabs.nlp.SparkNLP.start(false, false, false, "12G");

        Dataset<Row> data = spark.createDataset(text, Encoders.STRING()).toDF("text");

        DocumentAssembler document = new DocumentAssembler();
        document.setInputCol("text");
        document.setOutputCol("document");
        document.setCleanupMode("disabled");

        Tokenizer tokenizer = new Tokenizer();
        tokenizer.setInputCols(new String[] {"document"});
        tokenizer.setOutputCol("token");

        SentenceDetector sentenceDetector = new SentenceDetector();
        sentenceDetector.setInputCols(new String[]{"document"});
        sentenceDetector.setOutputCol("sentence");

        PerceptronModel posDetector = PerceptronModel.pretrained();

        WordEmbeddingsModel embeddings = WordEmbeddingsModel.pretrained();
        embeddings.setInputCols(new String[]{"document", "token"});
        embeddings.setOutputCol("embeddings");

        RecursivePipeline pipeline = new RecursivePipeline();
        pipeline.setStages(new PipelineStage[] {document, tokenizer, sentenceDetector, posDetector /* dependencyParserModel, embeddings,nerTagger */});
        PipelineModel pipelineModel = pipeline.fit(data);

        Dataset<Row> transformed = pipelineModel.transform(data);

        text.clear();

        Object res = transformed.collect();
        Row[] rows = (Row[]) res;

        List<String> posSentances = new ArrayList<>();

        //POS
        for(Row row:rows) {

            WrappedArray partOfSpeechResult = (WrappedArray) row.get(4);
            Row[] partOfSpeechRow = (Row[]) partOfSpeechResult.array();
            String partOfSpeechSentance = "";
            for (Row partOfSpeechVal:partOfSpeechRow) {
                String partOfSpeech = partOfSpeechVal.get(3).toString();
                String word = (((scala.collection.immutable.Map<String, String>) partOfSpeechVal.get(4)).get("word").get());
                if(partOfSpeech.equalsIgnoreCase("PRP") || partOfSpeech.equalsIgnoreCase("PRP$")  || word.toLowerCase().startsWith("i'")) {
                    partOfSpeechSentance = partOfSpeechSentance + word + " ";
                } else {
                    //partOfSpeechSentance = partOfSpeechSentance + partOfSpeech + " ";
                }
            }
            if(partOfSpeechSentance.length()==0) {
                partOfSpeechSentance = (String) row.get(0);
            }
            posSentances.add(partOfSpeechSentance.trim());
        }
        String rpFilename = "/" + baseResultsFileName + "rp.txt";
        FileUtils.writeLines(new File(System.getProperty("user.dir") + rpFilename), posSentances);
        return posSentances;

    }

}
