package co.uk.fues.freudbot.NLP;

import co.uk.fues.freudbot.data.TweetCleaner;
import com.johnsnowlabs.nlp.DocumentAssembler;
import com.johnsnowlabs.nlp.RecursivePipeline;
import com.johnsnowlabs.nlp.SparkNLP;
import com.johnsnowlabs.nlp.annotators.Tokenizer;
import com.johnsnowlabs.nlp.annotators.ner.dl.NerDLModel;
import com.johnsnowlabs.nlp.annotators.parser.dep.DependencyParserModel;
import com.johnsnowlabs.nlp.annotators.parser.typdep.TypedDependencyParserApproach;
import com.johnsnowlabs.nlp.annotators.parser.typdep.TypedDependencyParserModel;
import com.johnsnowlabs.nlp.annotators.pos.perceptron.PerceptronModel;
import com.johnsnowlabs.nlp.annotators.sbd.pragmatic.SentenceDetector;
import com.johnsnowlabs.nlp.embeddings.WordEmbeddingsModel;
import org.apache.commons.io.FileUtils;
import org.apache.spark.ml.PipelineModel;
import org.apache.spark.ml.PipelineStage;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Encoders;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;
import scala.collection.mutable.WrappedArray;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

public class TemporalDepParser {

    public static String joinedResultsFileName = "/joinedtmp.txt";
    public static String joinedResultsConvoFileName = "/joinedtmpconvo.txt";

    public static void main(String args[]) throws IOException {

        LinkedList<String> text = getData(TweetCleaner.cleanResultsFileName);
//        LinkedList<String> text = new LinkedList<>();
//        text.add("I called you a bot because you are a bot and all you have following you are bots. In the alternative, you are a Cabal puppet who does not want to be suicided any time soon.");
//        text.add("Oh stfu. You can’t really believe this. Either you are a bot a troll or a moron of gigantic proportions. Your orange cult leader and his sycophants are too chicken shit to w subpoenas, knowing they’d have to lie or be charged w a slew of fed crimes. My $ on click farm");
//        text.add("You are a bot with an agenda and are not interested in facts. Have a great day.");
//        text.add("And BTW, \"chosen\" does not mean \"better than\" in Jewish thrology. Your canned insults are so ludicrous, I am convinced you are a bot. Waste of time. Bye bye.");

        batchRun(text, "/tweetBatchResults/res/");
        stitch("/tweetBatchResults/res/", joinedResultsFileName);


        batchRun(text, "/tweetconvoBatchResults/res/");
        stitch("/tweetBatchResults/res/", joinedResultsConvoFileName);
        //postProcessPhenomise();

    }

    public static void batchRun(LinkedList<String> text, String resultsName) throws IOException {
        LinkedList<String> batch = new LinkedList<>();
        LinkedList<String> batch2 = new LinkedList<>();
        int iter = 1;
        for(int i = 0; i< text.size(); i++) {
            batch.add(text.get(i));
            if (batch.size() >= 5000) {
                batch2.addAll(run(batch, resultsName + iter));
                batch.clear();
                iter++;
            }
        }
        batch2.addAll(run(batch, resultsName + iter));
        batch2.size();
    }

    public static void stitch(String batchDir, String joinedResultsFileName) throws IOException {
        LinkedList<String> batch = new LinkedList<>();
        for (int i=1;i<12;i++) {
            batch.addAll(FileUtils.readLines(new File(System.getProperty("user.dir") + batchDir + i + "temporal.txt")));
        }
        FileUtils.writeLines(new File(System.getProperty("user.dir") + joinedResultsFileName), batch);
    }

    public static List<String> run(LinkedList<String> text, String baseResultsFileName) throws IOException {
        SparkSession spark = SparkNLP.start(false, false, false, "12G");

        Dataset<Row> data = spark.createDataset(text, Encoders.STRING()).toDF("text");

        DocumentAssembler document = new DocumentAssembler();
        document.setInputCol("text");
        document.setOutputCol("document");
        document.setCleanupMode("disabled");

        Tokenizer tokenizer = new Tokenizer();
        tokenizer.setInputCols(new String[] {"document"});
        tokenizer.setOutputCol("token");

        SentenceDetector sentenceDetector = new SentenceDetector();
        sentenceDetector.setInputCols(new String[]{"document"});
        sentenceDetector.setOutputCol("sentence");

        PerceptronModel posDetector = PerceptronModel.pretrained();

        DependencyParserModel dependencyParserModel = DependencyParserModel.pretrained();
        TypedDependencyParserModel typedDependencyParserApproach = TypedDependencyParserModel.pretrained();

        WordEmbeddingsModel embeddings = WordEmbeddingsModel.pretrained();
        embeddings.setInputCols(new String[]{"document", "token"});
        embeddings.setOutputCol("embeddings");

        NerDLModel nerTagger = NerDLModel.pretrained();

        RecursivePipeline pipeline = new RecursivePipeline();
        pipeline.setStages(new PipelineStage[] {document, tokenizer, sentenceDetector, posDetector, dependencyParserModel, typedDependencyParserApproach /*, embeddings,nerTagger */});
        PipelineModel pipelineModel = pipeline.fit(data);
        System.out.println("pipe");

        Dataset<Row> transformed = pipelineModel.transform(data);

        text.clear();

        Object res = transformed.collect();
        Row[] rows = (Row[]) res;

        List<String> posSentances = new ArrayList<>();
        List<String> depSentances = new ArrayList<>();
        for(Row row:rows) {

            WrappedArray partOfSpeechResult = (WrappedArray) row.get(4);
            Row[] partOfSpeechRow = (Row[]) partOfSpeechResult.array();
            String partOfSpeechSentance = "";
            for (int i=0;i<partOfSpeechRow.length;i++) {
                Row partOfSpeechVal = partOfSpeechRow[i];
                String partOfSpeech = partOfSpeechVal.get(3).toString();
                String word = (((scala.collection.immutable.Map<String, String>) partOfSpeechVal.get(4)).get("word").get());
                String nextWord = i<partOfSpeechRow.length-1 ? (((scala.collection.immutable.Map<String, String>) partOfSpeechRow[i+1].get(4)).get("word").get()) : null;
                String prevSubject = i>0? (((scala.collection.immutable.Map<String, String>) partOfSpeechRow[i-1].get(4)).get("word").get()) : null;
                String tense = null;
                if (word.equalsIgnoreCase("will") || word.equalsIgnoreCase("shall") || (word.equalsIgnoreCase("going") &&  nextWord!=null && nextWord.equalsIgnoreCase("to"))) {
                    // future
                    tense = "future";
                } else if(partOfSpeech.equalsIgnoreCase("VBD")) {
                    // past
                    tense = "past";
                } else if(partOfSpeech.equalsIgnoreCase("VBG")) {
                    // present participle
                    tense = "present";
                } else if(partOfSpeech.equalsIgnoreCase("VBN")) {
                    // Verb, past participle
                    tense = "past";
                } else if(partOfSpeech.equalsIgnoreCase("VBP")) {
                    // Verb, non-3rd person singular present
                    tense = "present";
                }  else if(partOfSpeech.equalsIgnoreCase("VBZ")) {
                    // Verb, 3rd person singular present
                    tense = "present";
                }

                if(tense!=null && (prevSubject!=null && ( prevSubject.equalsIgnoreCase("you") || prevSubject.contains("you'") || prevSubject.equalsIgnoreCase("i") || prevSubject.contains("i'")  || prevSubject.equalsIgnoreCase("we")  || prevSubject.contains("we'")  || prevSubject.equalsIgnoreCase("they")  || prevSubject.contains("they'")))) {
                    partOfSpeechSentance = partOfSpeechSentance + prevSubject.toLowerCase() + "-" + tense + " ";
                }
            }
            if(partOfSpeechSentance.length()==0) {
                partOfSpeechSentance = "none";
            }
            posSentances.add(partOfSpeechSentance.trim());
        }
        String postResultsFileName = "/" + baseResultsFileName + "temporal.txt";
        FileUtils.writeLines(new File(System.getProperty("user.dir") + postResultsFileName), posSentances);
        return posSentances;

        //DEPS
//        for(Row row:rows) {
//            WrappedArray dependencyResult = (WrappedArray) row.get(5);
//            Row[] dependencyRow = (Row[]) dependencyResult.array();
//            String dependencySentance = "";
//            for (Row dependencyVal:dependencyRow) {
//                String dependency = dependencyVal.get(3).toString();
//                if(!dependency.equals("ROOT")) {
//                    dependencySentance = dependencySentance + dependency + " ";
//                }
//            }
//            depSentances.add(dependencySentance);
//        }
//        String depResultsFileName = "/" + baseResultsFileName + "dep.txt";
//        FileUtils.writeLines(new File(System.getProperty("user.dir") + depResultsFileName), posSentances);
//
//        return depSentances;

    }

    public static LinkedList<String> getData(String cleanResultsFileName) throws IOException {
        LinkedList<String> text = new LinkedList<String>();
        text.addAll(FileUtils.readLines(new File(System.getProperty("user.dir") + cleanResultsFileName)));
        return text;
    }


//        RecursivePipeline pipeline = new RecursivePipeline();
//        pipeline.setStages(new PipelineStage[] {document, tokenizer});
//        PipelineModel pipelineModel = pipeline.fit(data);

//        Dataset<Row> transformed = pipelineModel.transform(data);
//        transformed.show();
//
//        PretrainedPipeline pretrained = new PretrainedPipeline("explain_document_dl");
//        pretrained.transform(data).show();

//        LemmatizerModel lemmatizer = (LemmatizerModel) LemmatizerModel.pretrained("lemma_antbnc");
//        lemmatizer.setInputCols(new String[] {"token"});
//        lemmatizer.setOutputCol("lemma");
//        lemmatizer.transform(transformed).show();


}
