package co.uk.fues.freudbot.cluster;

import com.google.common.util.concurrent.AtomicDouble;
import com.google.gson.Gson;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.time.StopWatch;
import org.deeplearning4j.clustering.cluster.Cluster;
import org.deeplearning4j.clustering.cluster.ClusterSet;
import org.deeplearning4j.clustering.cluster.Point;
import org.deeplearning4j.clustering.cluster.PointClassification;
import org.deeplearning4j.clustering.kmeans.KMeansClustering;
import org.deeplearning4j.models.paragraphvectors.ParagraphVectors;
import org.deeplearning4j.models.word2vec.wordstore.inmemory.InMemoryLookupCache;
import org.deeplearning4j.text.documentiterator.LabelsSource;
import org.deeplearning4j.text.sentenceiterator.BasicLineIterator;
import org.deeplearning4j.text.sentenceiterator.SentenceIterator;
import org.deeplearning4j.text.tokenization.tokenizerfactory.DefaultTokenizerFactory;
import org.deeplearning4j.text.tokenization.tokenizerfactory.TokenizerFactory;
import org.nd4j.linalg.api.ndarray.INDArray;
import org.nd4j.linalg.factory.Nd4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * This is example code for dl4j ParagraphVectors implementation. In this example we build distributed representation of all sentences present in training corpus.
 * However, you still use it for training on labelled documents, using sets of LabelledDocument and LabelAwareIterator implementation.
 * <p>
 * *************************************************************************************************
 * PLEASE NOTE: THIS EXAMPLE REQUIRES DL4J/ND4J VERSIONS >= rc3.8 TO COMPILE SUCCESSFULLY
 * *************************************************************************************************
 *
 * @author raver119@gmail.com
 */
public class KMeans {

    private static final Logger log = LoggerFactory.getLogger(KMeans.class);

    private static List<String> stopwords = new ArrayList<>();
    private static List<String> filterWords = new ArrayList<>();
    public static void main(String[] args) throws Exception {
//        String datafilepath = "/raw_sentences.txt"; //This has ~99000 single-sentence paragraphs/docs
//        ClassPathResource resource = new ClassPathResource(datafilepath);
//        File file = resource.getFile();
        File results = new File(System.getProperty("user.dir") + "/stopwords_en.txt");
        stopwords = FileUtils.readLines(results);
        stopwords.add("people");
        stopwords.add("person");
        stopwords.add("you’re");
        stopwords.add("don’t");
        stopwords.add("can’t");
        stopwords.add("account");
        stopwords.add("bot");

        filterWords.add("forex");
        filterWords.add("developer");
        filterWords.add("trading");
        filterWords.add("russian");
        filterWords.add("trump");
        filterWords.add("russia");

        String cleanResultsFileName = "/tweetresultsplainclean2.txt";
        KMeans kMeans = new KMeans();
//        String nerFileName = "/tweetresultsphemnome.txt";
//        ParagraphVectors vec = getParagraphVectors(nerFileName);
        String resultsDir = "/results/";
//        kMeans.method(vec, nerFileName, 10);
        kMeans.showClusters(resultsDir, 5);
        kMeans.showClusters(resultsDir, 10);
        kMeans.showClusters(resultsDir, 20);

    }


    private String removePhrase(String phrase) {
        String lc = phrase.toLowerCase();
        int phraseIndex = lc.indexOf("you are a bot");
        if(phraseIndex>-1) {
            String removed = phrase.substring(0, phraseIndex) + phrase.substring(phraseIndex + 13);
            return removed;
        }
        return phrase;
    }

    // TAG WORDS AS POS/TYPE
    // RANK WORD COUNT

    public void showClusters(String resultsDirName, int clusterCount) throws IOException {
        Map<String, List<String>> clusters = new HashMap<>();
        Map<String, Integer> allWords = new HashMap<>();
        Map<String, Map<String, Integer>> allWordsPerCluster = new HashMap<>();

        Gson gson = new Gson();

        Map<String, List<ClusterResult>> results = new HashMap<>();

        for (File file : new File(System.getProperty("user.dir") + resultsDirName + clusterCount).listFiles()) {

            try {
                List<ClusterResult> cresults = FileUtils.readLines(file).stream().map(line -> gson.fromJson(line, ClusterResult.class)).collect(Collectors.toList());
                Collections.sort(cresults);
                Collections.reverse(cresults);
                cresults = cresults.subList(0, 200);
                results.put(file.getName(), cresults);
            } catch (Exception e) {
                //e.printStackTrace();
            }
        }

        for (Map.Entry<String, List<ClusterResult>> cresults : results.entrySet()) {

            Map<String, Integer> words = new HashMap<>();
            try {
                cresults.getValue().forEach(tweetR -> {

                    try {
                        String tweet = tweetR.getTweet();//tweetLines.get(Integer.parseInt(line));
                        //System.out.println(tweet);

                        Arrays.stream(tweet.split(" ")).forEach(word -> {
                            words.putIfAbsent(word.toLowerCase(), 0);
                            words.put(word.toLowerCase(), words.get(word.toLowerCase()) + 1);

                            allWords.putIfAbsent(word.toLowerCase(), 0);
                            allWords.put(word.toLowerCase(), allWords.get(word.toLowerCase()) + 1);
                        });

                        allWordsPerCluster.put(cresults.getKey(), words);
                    } catch (Exception e2) {
                        e2.printStackTrace();
                    }
                });
            } catch (Exception e) {
                e.printStackTrace();
            }

            stopwords.forEach(stop -> {
                words.remove(stop);
                words.remove(stop.toLowerCase());
                allWords.remove(stop);
                allWords.remove(stop.toLowerCase());
            });
        }

        Map<String, Integer> allWordsTrimmed = new HashMap<>();

        allWords.entrySet().forEach(stringIntegerEntry -> {
            String word = stringIntegerEntry.getKey();
            int count = stringIntegerEntry.getValue();
            if(word.length()>4 && count>15) {
                allWordsTrimmed.put(stringIntegerEntry.getKey(), stringIntegerEntry.getValue());
            }
        });

        Map<String, Map<String, Integer>> allWordsPerClusterTop = new HashMap<>();

        allWordsTrimmed.keySet().forEach(topWord -> {

            AtomicDouble count = new AtomicDouble();
            AtomicReference<String> topCluster = new AtomicReference<>();
            allWordsPerCluster.entrySet().forEach(clusterTopWords -> {
                String file = clusterTopWords.getKey();

                if(clusterTopWords.getValue().containsKey(topWord)) {
                    int allClusterCount = clusterTopWords.getValue().get(topWord);
                    double clusterPortion = ((double)allClusterCount/(double)allWordsPerCluster.get(file).size()) * 10000;
                    if(clusterPortion> count.get()) {
                        count.set(clusterPortion);
                        topCluster.set(file);
                    }
                }
            });
            allWordsPerClusterTop.putIfAbsent(topCluster.get(), new HashMap<>());
            if(count.get()>15) {
                allWordsPerClusterTop.get(topCluster.get()).put(topWord, (int) count.get());
            }
        });

        Map<String, List<ClusterResult>> results2 = new HashMap<>();

        System.out.println("*******" + clusterCount + "*******");
        int i=0;
        for (File file : new File(System.getProperty("user.dir") + resultsDirName + clusterCount).listFiles()) {

            try {
                List<ClusterResult> cresults = FileUtils.readLines(file).stream().map(line -> gson.fromJson(line, ClusterResult.class)).collect(Collectors.toList());
                Collections.sort(cresults);
                Collections.reverse(cresults);
                cresults = cresults.subList(0, 20);
                i++;
                System.out.println("******* cluster " + i);
                cresults.forEach(clusterResult -> System.out.println(clusterResult.getTweet()));
                results2.put(file.getName(), cresults);
            } catch (Exception e) {
                //e.printStackTrace();
            }
        }

        clusters.size();
    }

    public void method(ParagraphVectors vec, String cleanResultsFileName, String resultsDirName, int clusterCount) throws IOException {
        StopWatch sw = new StopWatch();

        //1. create a kmeanscluster instance
        int maxIterationCount = 5;
        String distanceFunction = "cosinesimilarity";
        KMeansClustering kmc = KMeansClustering.setup(clusterCount, maxIterationCount, distanceFunction);

        //2. iterate over rows in the paragraphvector and create a List of paragraph vectors
        List<INDArray> vectors = new ArrayList<>();

        // EXTRACT WORDS FROM HERE
        for (String word : vec.vocab().words()) {
//            vec.getWordVectorMatrix(word).
//            if(word.contains("DOC_")) {
                vectors.add(vec.getWordVectorMatrix(word));
//            }
        }

        System.out.println(vectors.size() + " vectors extracted to create Point list");
        List<Point> pointsLst = Point.toPoints(vectors);
        // MAP THEM
//        Map<String, Point> pointsLst2 = new HashMap<>();
        System.out.println(pointsLst.size() + " Points created out of " + vectors.size() + " vectors");

        System.out.println("Start Clustering " + pointsLst.size() + " points/docs");
        sw.reset();
        sw.start();
        ClusterSet cs = kmc.applyTo(pointsLst);
        sw.stop();
        System.out.println("Time taken to run clustering on " + vectors.size() + " paragraphVectors: " + sw.getTime());

        System.out.println("Finish Clustering");

        List<Cluster> clsterLst = cs.getClusters();

        System.out.println("\nCluster Centers:");
        for (Cluster c : clsterLst) {
            Point center = c.getCenter();
            System.out.println(center.getLabel());
        }

        log.info("Trying to classify a point that was used for generating the Clusters");

        Map<String, List<Integer>> clusterIdToLines = new HashMap<>();
        Map<String, List<String>> clusterIdToLinesDistance = new HashMap<>();

        Map<String, List<Double>> avgD = new HashMap<>();
        Gson gson = new Gson();

        File results = new File(System.getProperty("user.dir") + cleanResultsFileName);
        List<String> tweetLines = FileUtils.readLines(results);

        for (int i = 0; i < vectors.size(); i++) {
            try {
                double[] nesVec = vec.getWordVector("DOC_" + i);

                if (nesVec != null) {
                    Point newpoint = new Point("myid" + i, "mylabel", nesVec);
                    PointClassification pc = cs.classifyPoint(newpoint);
                    String cid = pc.getCluster().getCenter().getId();
                    double dis = pc.getDistanceFromCenter();
//                    if(dis>-0.002 && dis<0.002) {
                        //System.out.println(dis);
                    clusterIdToLines.putIfAbsent(cid, new ArrayList<>());
                    clusterIdToLines.get(cid).add(i);

                    ClusterResult result = new ClusterResult();
                    result.setLine(i);
                    result.setDistance(dis);
                    result.setTweet(tweetLines.get(i));
                    clusterIdToLinesDistance.putIfAbsent(cid, new ArrayList<>());
                    clusterIdToLinesDistance.get(cid).add(gson.toJson(result));

                    avgD.putIfAbsent(cid, new ArrayList<>());
                    avgD.get(cid).add(Math.abs(dis));
//                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

        }

        avgD.entrySet().forEach(stringListEntry -> {
            String cid = stringListEntry.getKey();
            double dist = stringListEntry.getValue().stream().reduce(Double::sum).get() / stringListEntry.getValue().size();

            System.out.println(cid + "   " + dist);
        });

        File dire = new File(System.getProperty("user.dir") + resultsDirName + clusterCount);
        if(dire.exists()) {
            FileUtils.forceDelete(dire);
        }
//        File dirre = new File(System.getProperty("user.dir") + "/results-lines/");
//        if(dirre.exists()) {
//            FileUtils.forceDelete(dirre);
//        }

//        clusterIdToLines.keySet().forEach(clu -> {
//            try {
//                FileUtils.writeLines(new File(System.getProperty("user.dir") + "/results/" + clusterCount + "/" + clu + ".txt"), clusterIdToLines.get(clu));
//            } catch (IOException e) {
//                e.printStackTrace();
//            }
//        });
        clusterIdToLinesDistance.keySet().forEach(clu -> {
            try {
                FileUtils.writeLines(new File(System.getProperty("user.dir") + "/results/"  + clusterCount + "/" + clu + ".txt"), clusterIdToLinesDistance.get(clu));
            } catch (IOException e) {
                e.printStackTrace();
            }
        });

        System.out.println("\nEnd Test");
        vectors = null;
        pointsLst = null;
    }

    public static ParagraphVectors getParagraphVectors(String resultsFileName) throws FileNotFoundException {
        File file = new File(System.getProperty("user.dir") + resultsFileName);
        SentenceIterator iter = new BasicLineIterator(file);
        InMemoryLookupCache cache = new InMemoryLookupCache();
        TokenizerFactory t = new DefaultTokenizerFactory();
        //t.setTokenPreProcessor(new CommonPreprocessor());

        LabelsSource source = new LabelsSource("DOC_");
        Nd4j.getRandom().setSeed(7);

        ParagraphVectors vec = new ParagraphVectors.Builder()
                .seed(7)
                .minWordFrequency(3)
                .iterations(3)
                .epochs(20)
                .layerSize(100)  /*length of a paragraph vector*/
                .learningRate(0.025)
                .labelsSource(source)
                .windowSize(5)
                .iterate(iter)
                .trainWordVectors(true)
                .vocabCache(cache)
                .tokenizerFactory(t)
                .sampling(0)
                .build();

        vec.fit();
        return vec;
    }
}
