package co.uk.fues.freudbot.data;

import co.uk.fues.freudbot.NLP.NLP;
import co.uk.fues.freudbot.ingest.Tweet;
import com.google.gson.Gson;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

public class TweetCleaner {

    private static final Logger log = LoggerFactory.getLogger(TweetCleaner.class);

    private static List<String> stopwords = new ArrayList<>();
    private static List<String> filterWords = new ArrayList<>();
    public static String resultsFileName = "/tweetresultsplain2.txt";
    public static String cleanResultsFileName = "/tweetresultsplainclean2.txt";
    public static String cleanResultsConvoFileName = "/tweetconvoresultsplainclean2.txt";

    public static void main(String[] args) throws Exception {
//        String datafilepath = "/raw_sentences.txt"; //This has ~99000 single-sentence paragraphs/docs
//        ClassPathResource resource = new ClassPathResource(datafilepath);
//        File file = resource.getFile();
        File results = new File(System.getProperty("user.dir") + "/stopwords_en.txt");
        stopwords = FileUtils.readLines(results);
        stopwords.add("people");
        stopwords.add("person");
        stopwords.add("you’re");
        stopwords.add("don’t");
        stopwords.add("can’t");
        stopwords.add("account");
        stopwords.add("bot");


        TweetCleaner cleaner = new TweetCleaner();
        cleaner.makeLines(resultsFileName, "/data");
        cleaner.cleanFile(resultsFileName, cleanResultsFileName);
    }

    public void makeLines (String dirName, String resultsFilePlainName) throws IOException {
        File dir = new File(System.getProperty("user.dir") + dirName);
        Gson gson = new Gson();
        List<String> tweets = new ArrayList<>();
        Map<String, Integer> map = new HashMap<>();
        Arrays.stream(dir.listFiles()).forEach(file -> {
            try {
                List<String> tws = FileUtils.readLines(file).stream().map(line -> gson.fromJson(line, Tweet.class).getText()).collect(Collectors.toList());
                tws.forEach(cid -> {
                    map.putIfAbsent(cid, 0);
                    map.put(cid, map.get(cid+1));
                });
                tweets.addAll(tws);
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
        File file2 = new File(System.getProperty("user.dir") + resultsFilePlainName);
        file2.createNewFile();
        Set<String> red = new HashSet<>(tweets);
        FileUtils.writeLines(file2, red);
    }

    public void cleanFile(String resultsFileName, String cleanResultsFileName) throws IOException {
        File results = new File(System.getProperty("user.dir") + resultsFileName);
        List<String> lines = FileUtils.readLines(results);
        AtomicInteger count = new AtomicInteger();
        AtomicInteger wordSize = new AtomicInteger();
        Set<String> set = lines.stream()
                .map(
                        tweet -> {
                            String[] words = tweet.split(" ");
                            String newTweet = "";
                            for (String word : words) {
                                if (!word.contains("@")) {
                                    newTweet = newTweet + word + " ";
                                }
//                                else {
//                                    newTweet = newTweet  + "@ ";
//                                }
                            }
                            String twt = newTweet.trim();
                            return twt;
                        })
                .filter(line ->
                {
                    List<String> words = Arrays.asList(line.split(" "));
                    words = words.stream().map(word -> word.toLowerCase()).collect(Collectors.toList());
                    boolean include = words.size() > 11;
                    if (include) {
                        count.getAndIncrement();
                        wordSize.set(wordSize.get() + words.size());
                    }
                    return include;
                })
                .collect(Collectors.toSet());

        Integer avg = wordSize.get() / count.get();
        System.out.println("tweet word count: " + avg);
        FileUtils.writeLines(new File(System.getProperty("user.dir") + cleanResultsFileName), removeIf(set));
    }
    public static LinkedList<String> removeIf(Collection<String> texts) throws IOException {
        //String cleanResultsFileName = "/tweetresultsplainclean2.txt";
        //LinkedList<String> texts = NLP.getData(cleanResultsFileName);
        List<String> notexts = texts.stream().filter(text -> !text.contains("if") && !text.contains("maybe")  && !text.contains("or") && !text.contains("assume") && !text.contains("suspect")&& !text.contains("hope") && !text.contains("think") && !text.contains("believe")).collect(Collectors.toList());
        notexts.forEach(System.out::println);
        return new LinkedList<>(notexts);

    }

    private String removePhrase(String phrase) {
        String lc = phrase.toLowerCase();
        int phraseIndex = lc.indexOf("you are a bot");
        if (phraseIndex > -1) {
            String removed = phrase.substring(0, phraseIndex) + phrase.substring(phraseIndex + 13);
            return removed;
        }
        return phrase;
    }

    private boolean isHasPhrase(List<String> words, boolean hasPhrase) {

        for (String f:filterWords) {
            if(words.contains(f) || words.contains("#"+f)) {
                hasPhrase = false;
                break;
            }
        }
        return hasPhrase;
    }

}